@@ -13,7 +13,7 @@ use 5.010001;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.840';
+our $VERSION = '1.841';
 
 ## no critic (Subroutines::ProhibitExplicitReturnUndef)
 
@@ -25,7 +25,7 @@ use MCE::Mutex ();
 use constant {
    MAX_DQ_DEPTH => 192,  # Maximum dequeue notifications
 
-   MUTEX_LOCKS  => 6,    # Number of mutex locks for 1st level defense
+   MUTEX_LOCKS  => 5,    # Number of mutex locks for 1st level defense
                          # against many workers waiting to dequeue
 };
 
@@ -50,13 +50,15 @@ sub _croak {
 our ($HIGHEST, $LOWEST, $FIFO, $LIFO, $LILO, $FILO) = (1, 0, 1, 0, 1, 0);
 my  ($PORDER, $TYPE, $FAST, $AWAIT) = ($HIGHEST, $FIFO, 0, 0);
 
-my %_valid_fields_new = map { $_ => 1 } qw( await fast porder queue type );
-
 my $LF = "\012"; Internals::SvREADONLY($LF, 1);
 my $_has_threads = $INC{'threads.pm'} ? 1 : 0;
 my $_tid = $_has_threads ? threads->tid() : 0;
 my $_reset_flg = 1;
 
+my %_valid_fields_new = map { $_ => 1 } qw(
+   await barrier fast porder queue type
+);
+
 sub CLONE {
    $_tid = threads->tid() if $_has_threads;
 }
@@ -104,14 +106,10 @@ sub new {
 
    # --------------------------------------------------------------------------
 
-   $_Q->{_await} = (exists $_argv{await} && defined $_argv{await})
-      ? $_argv{await} : $AWAIT;
-   $_Q->{_fast} = (exists $_argv{fast} && defined $_argv{fast})
-      ? $_argv{fast} : $FAST;
-   $_Q->{_porder} = (exists $_argv{porder} && defined $_argv{porder})
-      ? $_argv{porder} : $PORDER;
-   $_Q->{_type} = (exists $_argv{type} && defined $_argv{type})
-      ? $_argv{type} : $TYPE;
+   $_Q->{_await}  = defined $_argv{await}  ? $_argv{await}  : $AWAIT;
+   $_Q->{_fast}   = defined $_argv{fast}   ? $_argv{fast}   : $FAST;
+   $_Q->{_porder} = defined $_argv{porder} ? $_argv{porder} : $PORDER;
+   $_Q->{_type}   = defined $_argv{type}   ? $_argv{type}   : $TYPE;
 
    if (exists $_argv{queue}) {
       _croak('Queue: (queue) is not an ARRAY reference')
@@ -127,9 +125,10 @@ sub new {
    $_Q->{_init_pid} = $_has_threads ? $$ .'.'. $_tid : $$;
    $_Q->{_dsem} = 0 if $_Q->{_fast};
 
-   my $_caller = caller() eq 'MCE::Shared' ? caller(1) : caller();
+   my $_barrier = defined $_argv{barrier} ? $_argv{barrier} : 1;
+   my $_caller  = caller() eq 'MCE::Shared' ? caller(1) : caller();
 
-   if ($^O ne 'MSWin32' && $_tid == 0 && !$_Q->{_fast}) {
+   if ($^O ne 'MSWin32' && $_tid == 0 && !$_Q->{_fast} && $_barrier) {
       if ($_caller !~ /^MCE::/) {
          for my $_i (0 .. MUTEX_LOCKS - 1) {
             $_Q->{'_mutex_'.$_i} = MCE::Mutex->new( impl => 'Channel' );
@@ -140,12 +139,11 @@ sub new {
    MCE::Util::_sock_pair($_Q, qw(_qr_sock _qw_sock));
    MCE::Util::_sock_pair($_Q, qw(_ar_sock _aw_sock)) if $_Q->{_await};
 
-   if (exists $_argv{queue} && scalar @{ $_argv{queue} }) {
-      1 until syswrite($_Q->{_qw_sock}, $LF) || ($! && !$!{'EINTR'});
-   }
+   syswrite($_Q->{_qw_sock}, $LF)
+      if (exists $_argv{queue} && scalar @{ $_argv{queue} });
 
    MCE::Shared::Object::_reset(), $_reset_flg = ''
-      if $_reset_flg && $INC{'MCE/Shared/Server.pm'};
+      if ($_reset_flg && $INC{'MCE/Shared/Server.pm'});
 
    return $_Q;
 }
@@ -168,17 +166,9 @@ sub await {
 sub clear {
    my ($_Q) = @_;
 
-   if ($_Q->{_fast}) {
-      warn "Queue: (clear) is not allowed for fast => 1\n";
-   }
-   else {
-      if ($_Q->_has_data()) {
-         1 until sysread($_Q->{_qr_sock}, my($_b), 1) || ($! && !$!{'EINTR'});
-      }
-      %{ $_Q->{_datp} } = ();
-      @{ $_Q->{_datq} } = ();
-      @{ $_Q->{_heap} } = ();
-   }
+   %{ $_Q->{_datp} } = ();
+   @{ $_Q->{_datq} } = ();
+   @{ $_Q->{_heap} } = ();
 
    return;
 }
@@ -189,9 +179,7 @@ sub end {
    my ($_Q) = @_;
 
    if (!exists $_Q->{_ended}) {
-      if (!$_Q->{_nb_flag}) {
-         1 until syswrite($_Q->{_qw_sock}, $LF) || ($! && !$!{'EINTR'});
-      }
+      syswrite($_Q->{_qw_sock}, $LF) unless $_Q->{_nb_flag};
       $_Q->{_ended} = undef;
    }
 
@@ -210,7 +198,7 @@ sub enqueue {
       return;
    }
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      1 until syswrite($_Q->{_qw_sock}, $LF) || ($! && !$!{'EINTR'});
+      syswrite($_Q->{_qw_sock}, $LF);
    }
 
    push @{ $_Q->{_datq} }, @_;
@@ -233,7 +221,7 @@ sub enqueuep {
       return;
    }
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      1 until syswrite($_Q->{_qw_sock}, $LF) || ($! && !$!{'EINTR'});
+      syswrite($_Q->{_qw_sock}, $LF);
    }
 
    $_Q->_enqueuep($_p, @_);
@@ -248,7 +236,7 @@ sub dequeue {
    my ($_Q, $_cnt) = @_;
    my (@_items, $_buf, $_next, $_pending);
 
-   1 until sysread($_Q->{_qr_sock}, $_next, 1) || ($! && !$!{'EINTR'});
+   MCE::Util::_sysread($_Q->{_qr_sock}, $_next, 1);
 
    if (defined $_cnt && $_cnt ne '1') {
       _croak('Queue: (dequeue count argument) is not valid')
@@ -277,7 +265,7 @@ sub dequeue {
          if ($_pending) {
             $_pending = MAX_DQ_DEPTH if ($_pending > MAX_DQ_DEPTH);
             for my $_i (1 .. $_pending) {
-               1 until syswrite($_Q->{_qw_sock}, $LF) || ($! && !$!{'EINTR'});
+               syswrite($_Q->{_qw_sock}, $LF);
             }
          }
          $_Q->{_dsem} = $_pending;
@@ -288,13 +276,11 @@ sub dequeue {
    }
    else {
       # Otherwise, never to exceed one byte in the channel
-      if ($_Q->_has_data()) {
-         1 until syswrite($_Q->{_qw_sock}, $LF) || ($! && !$!{'EINTR'});
-      }
+      syswrite($_Q->{_qw_sock}, $LF) if $_Q->_has_data();
    }
 
    if (exists $_Q->{_ended} && !$_Q->_has_data()) {
-      1 until syswrite($_Q->{_qw_sock}, $LF) || ($! && !$!{'EINTR'});
+      syswrite($_Q->{_qw_sock}, $LF);
    }
 
    $_Q->{_nb_flag} = 0;
@@ -315,7 +301,7 @@ sub dequeue_nb {
    }
 
    if (!$_Q->{_nb_flag} && $_Q->_has_data()) {
-      1 until sysread($_Q->{_qr_sock}, my($_b), 1) || ($! && !$!{'EINTR'});
+      MCE::Util::_sysread($_Q->{_qr_sock}, my($_b), 1);
    }
 
    if (defined $_cnt && $_cnt ne '1') {
@@ -374,7 +360,7 @@ sub insert {
       return;
    }
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      1 until syswrite($_Q->{_qw_sock}, $LF) || ($! && !$!{'EINTR'});
+      syswrite($_Q->{_qw_sock}, $LF);
    }
 
    if (abs($_i) > scalar @{ $_Q->{_datq} }) {
@@ -422,7 +408,7 @@ sub insertp {
       return;
    }
    if (!$_Q->{_nb_flag} && !$_Q->_has_data()) {
-      1 until syswrite($_Q->{_qw_sock}, $LF) || ($! && !$!{'EINTR'});
+      syswrite($_Q->{_qw_sock}, $LF);
    }
 
    if (exists $_Q->{_datp}->{$_p} && scalar @{ $_Q->{_datp}->{$_p} }) {
@@ -710,7 +696,7 @@ sub _heap_insert_high {
          $_Q->{_tsem} = $_t;
 
          if ($_Q->pending() <= $_t) {
-            1 until syswrite($_Q->{_aw_sock}, $LF) || ($! && !$!{'EINTR'});
+            syswrite($_Q->{_aw_sock}, $LF);
          } else {
             $_Q->{_asem} += 1;
          }
@@ -759,7 +745,7 @@ sub _heap_insert_high {
                if ($_pending) {
                   $_pending = MAX_DQ_DEPTH if ($_pending > MAX_DQ_DEPTH);
                   for my $_i (1 .. $_pending) {
-                     1 until syswrite($_Q->{_qw_sock}, $LF) || ($! && !$!{'EINTR'});
+                     syswrite($_Q->{_qw_sock}, $LF);
                   }
                }
                $_Q->{_dsem} = $_pending;
@@ -770,13 +756,11 @@ sub _heap_insert_high {
          }
          else {
             # Otherwise, never to exceed one byte in the channel
-            if ($_Q->_has_data()) {
-               1 until syswrite($_Q->{_qw_sock}, $LF) || ($! && !$!{'EINTR'});
-            }
+            syswrite($_Q->{_qw_sock}, $LF) if $_Q->_has_data();
          }
 
          if (exists $_Q->{_ended} && !$_Q->_has_data()) {
-            1 until syswrite($_Q->{_qw_sock}, $LF) || ($! && !$!{'EINTR'});
+            syswrite($_Q->{_qw_sock}, $LF);
          }
 
          if ($_cnt) {
@@ -784,7 +768,7 @@ sub _heap_insert_high {
             print {$_DAU_R_SOCK} length($_buf).'1'.$LF, $_buf;
          }
          elsif (defined $_buf) {
-            if (!looks_like_number($_buf) && !ref($_buf)) {
+            if (!ref($_buf)) {
                print {$_DAU_R_SOCK} length($_buf).'0'.$LF, $_buf;
             } else {
                $_buf = $_freeze->([ $_buf ]);
@@ -797,7 +781,7 @@ sub _heap_insert_high {
 
          if ($_Q->{_await} && $_Q->{_asem} && $_Q->pending() <= $_Q->{_tsem}) {
             for my $_i (1 .. $_Q->{_asem}) {
-               1 until syswrite($_Q->{_aw_sock}, $LF) || ($! && !$!{'EINTR'});
+               syswrite($_Q->{_aw_sock}, $LF);
             }
             $_Q->{_asem} = 0;
          }
@@ -819,14 +803,14 @@ sub _heap_insert_high {
          };
 
          if (!$_Q->{_nb_flag} && $_Q->_has_data()) {
-            1 until sysread($_Q->{_qr_sock}, my($_b), 1) || ($! && !$!{'EINTR'});
+            MCE::Util::_sysread($_Q->{_qr_sock}, my($_b), 1);
          }
 
          if ($_cnt == 1) {
             my $_buf = $_Q->_dequeue();
 
             if (defined $_buf) {
-               if (!looks_like_number($_buf) && !ref($_buf)) {
+               if (!ref($_buf)) {
                   print {$_DAU_R_SOCK} length($_buf).'0'.$LF, $_buf;
                } else {
                   $_buf = $_freeze->([ $_buf ]);
@@ -860,7 +844,7 @@ sub _heap_insert_high {
 
          if ($_Q->{_await} && $_Q->{_asem} && $_Q->pending() <= $_Q->{_tsem}) {
             for my $_i (1 .. $_Q->{_asem}) {
-               1 until syswrite($_Q->{_aw_sock}, $LF) || ($! && !$!{'EINTR'});
+               syswrite($_Q->{_aw_sock}, $LF);
             }
             $_Q->{_asem} = 0;
          }
@@ -902,7 +886,7 @@ use warnings;
 no warnings qw( threads recursion uninitialized numeric once );
 
 use constant {
-   MUTEX_LOCKS => 6,  # Number of mutex locks for 1st level defense
+   MUTEX_LOCKS => 5,  # Number of mutex locks for 1st level defense
                       # against many workers waiting to dequeue
 };
 
@@ -912,11 +896,11 @@ no overloading;
 
 my $_is_MSWin32 = ($^O eq 'MSWin32') ? 1 : 0;
 
-my ($_DAT_W_SOCK, $_DAU_W_SOCK, $_dat_ex, $_dat_un, $_chn, $_obj,
+my ($_DAT_LOCK, $_DAT_W_SOCK, $_DAU_W_SOCK, $_dat_ex, $_dat_un, $_chn, $_obj,
     $_freeze, $_thaw, $_pending);
 
 sub _init_queue {
-   ($_DAT_W_SOCK, $_DAU_W_SOCK, $_dat_ex, $_dat_un, $_chn, $_obj,
+   ($_DAT_LOCK, $_DAT_W_SOCK, $_DAU_W_SOCK, $_dat_ex, $_dat_un, $_chn, $_obj,
     $_freeze, $_thaw) = @_;
 
    return;
@@ -926,17 +910,22 @@ sub _req_queue {
    local $\ = undef if (defined $\);
    local $/ = $LF if ($/ ne $LF);
 
-   $_dat_ex->();
-   print({$_DAT_W_SOCK} $_[0].$LF . $_chn.$LF),
-   print({$_DAU_W_SOCK} $_[1]);
+   CORE::lock $_DAT_LOCK if $_is_MSWin32;
+   $_dat_ex->() if !$_is_MSWin32;
    $_[3]->{'_mutex_'.( $_chn % MUTEX_LOCKS )}->unlock() if $_[3];
 
+   print({$_DAT_W_SOCK} $_[0].$LF . $_chn.$LF),
+   print({$_DAU_W_SOCK} $_[1]);
+
    chomp(my $_len = <$_DAU_W_SOCK>);
-   $_dat_un->(), return if ($_len < 0);
+   if ($_len < 0) {
+      $_dat_un->() if !$_is_MSWin32;
+      return;
+   }
 
    my $_frozen = chop($_len);
    read $_DAU_W_SOCK, my($_buf), $_len;
-   $_dat_un->();
+   $_dat_un->() if !$_is_MSWin32;
 
    ($_[2] == 1)
       ? ($_frozen) ? $_thaw->($_buf)[0] : $_buf
@@ -959,7 +948,7 @@ sub await {
    _req1('O~QUA', $_id.$LF . $_t.$LF);
 
    MCE::Util::_sock_ready($_Q->{_ar_sock}) if $_is_MSWin32;
-   1 until sysread($_Q->{_ar_sock}, my($_b), 1) || ($! && !$!{'EINTR'});
+   MCE::Util::_sysread($_Q->{_ar_sock}, my($_b), 1);
 
    return;
 }
@@ -980,15 +969,13 @@ sub dequeue {
 
    if (exists $_Q->{'_mutex_0'}) {
       $_Q->{'_mutex_'.( $_chn % MUTEX_LOCKS )}->lock();
-
-      MCE::Util::_sock_ready($_Q->{_qr_sock}) if $_is_MSWin32;
-      1 until sysread($_Q->{_qr_sock}, my($_b), 1) || ($! && !$!{'EINTR'});
+      MCE::Util::_sysread($_Q->{_qr_sock}, my($_b), 1);
 
       _req_queue('O~QUD', $_id.$LF . $_cnt.$LF, $_cnt, $_Q);
    }
    else {
       MCE::Util::_sock_ready($_Q->{_qr_sock}) if $_is_MSWin32;
-      1 until sysread($_Q->{_qr_sock}, my($_b), 1) || ($! && !$!{'EINTR'});
+      MCE::Util::_sysread($_Q->{_qr_sock}, my($_b), 1);
 
       _req_queue('O~QUD', $_id.$LF . $_cnt.$LF, $_cnt, undef);
    }
@@ -1035,7 +1022,7 @@ MCE::Shared::Queue - Hybrid-queue helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Queue version 1.840
+This document describes MCE::Shared::Queue version 1.841
 
 =head1 DESCRIPTION
 
@@ -1103,12 +1090,12 @@ the shared-manager process, otherwise locally.
 
 =head1 API DOCUMENTATION
 
-=over 3
+=head2 MCE::Shared::Queue->new ( [ options ] )
 
-=item new ( [ options ] )
+=head2 MCE::Shared->queue ( [ options ] )
 
-Constructs a new object. Supported options are queue, porder, type, await, and
-fast.
+Constructs a new object. Supported options are queue, porder, type, await,
+barrier, and fast.
 
  # non-shared or local construction for use by a single process
 
@@ -1123,7 +1110,7 @@ fast.
  $q5 = MCE::Shared::Queue->new( type   => $MCE::Shared::Queue::FIFO );
  $q6 = MCE::Shared::Queue->new( type   => $MCE::Shared::Queue::LIFO );
 
- $q7 = MCE::Shared::Queue->new( await  => 1 );
+ $q7 = MCE::Shared::Queue->new( await  => 1, barrier => 0 );
  $q8 = MCE::Shared::Queue->new( fast   => 1 );
 
  # construction for sharing with other threads and processes
@@ -1140,19 +1127,23 @@ fast.
  $q5 = MCE::Shared->queue( type   => $MCE::Shared::Queue::FIFO );
  $q6 = MCE::Shared->queue( type   => $MCE::Shared::Queue::LIFO );
 
- $q7 = MCE::Shared->queue( await  => 1 );
+ $q7 = MCE::Shared->queue( await  => 1, barrier => 0 );
  $q8 = MCE::Shared->queue( fast   => 1 );
 
 The C<await> option, when enabled, allows workers to block (semaphore-like)
 until the number of items pending is equal or less than a threshold value.
 The C<await> method is described below.
 
-The C<fast> option speeds up dequeues and is not enabled by default. It is
-beneficial for queues not calling (->clear or ->dequeue_nb) and not altering
-the optional count value while running; e.g. ->dequeue($count). Basically,
-do not enable 'fast' if varying the count dynamically.
+On Unix platforms, C<barrier> mode (enabled by default) prevents many workers
+from dequeuing simultaneously to lessen overhead for the OS kernel. Specify 0
+to disable barrier mode and not allocate sockets. The barrier option has no
+effect if constructing the queue inside a thread or enabling C<fast>.
 
-=item await ( pending_threshold )
+The C<fast> option speeds up dequeues and is not enabled by default. It is
+beneficial for queues not calling (->dequeue_nb) and not altering the count
+value while running; e.g. ->dequeue($count).
+
+=head2 await ( pending_threshold )
 
 Waits until the queue drops down to threshold items. The C<await> method is
 beneficial when wanting to throttle worker(s) appending to the queue. Perhaps,
@@ -1195,13 +1186,13 @@ increasing too high. Below, the number of items pending will never go above 20.
     }
  };
 
-=item clear ( )
+=head2 clear ( )
 
 Clears the queue of any items.
 
  $q->clear;
 
-=item end ( )
+=head2 end ( )
 
 Stops the queue from receiving more items. Any worker blocking on C<dequeue>
 will be unblocked automatically. Subsequent calls to C<dequeue> will behave
@@ -1217,21 +1208,21 @@ depends on how many items workers dequeue at a time.
  $q->enqueue((undef) x ($N_workers * 2));  # $q->dequeue(2)  2 items
  $q->enqueue((undef) x ($N_workers * N));  # $q->dequeue(N)  N items
 
-=item enqueue ( item [, item, ... ] )
+=head2 enqueue ( item [, item, ... ] )
 
 Appends a list of items onto the end of the normal queue.
 
  $q->enqueue( 'foo' );
  $q->enqueue( 'bar', 'baz' );
 
-=item enqueuep ( priority, item [, item, ... ] )
+=head2 enqueuep ( priority, item [, item, ... ] )
 
 Appends a list of items onto the end of the priority queue with priority.
 
  $q->enqueue( $priority, 'foo' );
  $q->enqueue( $priority, 'bar', 'baz' );
 
-=item dequeue ( [ count ] )
+=head2 dequeue ( [ count ] )
 
 Returns the requested number of items (default 1) from the queue. Priority
 data will always dequeue first before any data from the normal queue.
@@ -1259,7 +1250,7 @@ which will block until the requested number of items are available.
     ...
  }
 
-=item dequeue_nb ( [ count ] )
+=head2 dequeue_nb ( [ count ] )
 
 Returns the requested number of items (default 1) from the queue. Like with
 dequeue, priority data will always dequeue first. This method is non-blocking
@@ -1268,7 +1259,7 @@ and returns C<undef> in the absence of data.
  $q->dequeue_nb( 2 );
  $q->dequeue_nb; # default 1
 
-=item insert ( index, item [, item, ... ] )
+=head2 insert ( index, item [, item, ... ] )
 
 Adds the list of items to the queue at the specified index position (0 is the
 head of the list). The head of the queue is that item which would be removed
@@ -1284,12 +1275,12 @@ by a call to dequeue.
  $q->insert(1, 'foo', 'bar');
  # Queue now contains: 1, 2, 3, 'foo', 'bar', 4
 
-=item insertp ( priority, index, item [, item, ... ] )
+=head2 insertp ( priority, index, item [, item, ... ] )
 
 Adds the list of items to the queue at the specified index position with
 priority. The behavior is similarly to C<$q->insert> otherwise.
 
-=item pending ( )
+=head2 pending ( )
 
 Returns the number of items in the queue. The count includes both normal
 and priority data. Returns C<undef> if the queue has been ended, and there
@@ -1302,7 +1293,7 @@ are no more items in the queue.
  print $q->pending(), "\n";
  # Output: 4
 
-=item peek ( [ index ] )
+=head2 peek ( [ index ] )
 
 Returns an item from the normal queue, at the specified index, without
 dequeuing anything. It defaults to the head of the queue if index is not
@@ -1321,13 +1312,13 @@ call to dequeue. Negative index values are supported, similarly to arrays.
  print $q->peek(1), ' ', $q->peek(-2), "\n";
  # Output: 4 2
 
-=item peekp ( priority [, index ] )
+=head2 peekp ( priority [, index ] )
 
 Returns an item from the queue with priority, at the specified index, without
 dequeuing anything. It defaults to the head of the queue if index is not
 specified. The behavior is similarly to C<$q->peek> otherwise.
 
-=item peekh ( [ index ] )
+=head2 peekh ( [ index ] )
 
 Returns an item from the head of the heap or at the specified index.
 
@@ -1347,7 +1338,7 @@ Returns an item from the head of the heap or at the specified index.
  print $q->peekh(0), "\n";
  # Output: 4
 
-=item heap ( )
+=head2 heap ( )
 
 Returns an array containing the heap data. Heap data consists of priority
 numbers, not the data.
@@ -1358,27 +1349,25 @@ numbers, not the data.
  @h = $q->heap;   # $MCE::Shared::Queue::LOWEST
  # Heap contains: 4, 5, 6
 
-=back
-
 =head1 ACKNOWLEDGMENTS
 
 =over 3
 
-=item L<List::BinarySearch>
+=item * L<List::BinarySearch>
 
 The bsearch_num_pos method was helpful for accommodating the highest and lowest
 order in MCE::Shared::Queue.
 
-=item L<POE::Queue::Array>
+=item * L<POE::Queue::Array>
 
 For extra optimization, two if statements were adopted for checking if the item
 belongs at the end or head of the queue.
 
-=item L<List::Priority>
+=item * L<List::Priority>
 
 MCE::Shared::Queue supports both normal and priority queues.
 
-=item L<Thread::Queue>
+=item * L<Thread::Queue>
 
 Thread::Queue is used as a template for identifying and documenting the methods.
 MCE::Shared::Queue is not fully compatible due to supporting normal and priority
